@@ -5,6 +5,7 @@ import os
 import pickle
 import sqlite3
 import time
+from functools import lru_cache
 from typing import Optional
 
 import sqlglot
@@ -40,7 +41,7 @@ class SqliteDBExecutor(BaseSQLDBExecutor):
             raise FileNotFoundError(f"SQLite database file not found at {db_path}")
         uri = f"sqlite:///{db_path}?mode=ro&nolock=1&check_same_thread=false&immutable=1"
         # uri = f"sqlite:///{db_path}?check_same_thread=false&immutable=1"
-        logger.info(f"Connecting to SQLite database with URI {uri}")
+        logger.debug(f"Connecting to SQLite database with URI {uri}")
         engine = create_engine(uri,
                                # echo=True, echo_pool=True,
                                pool_size=20, max_overflow=40,
@@ -50,8 +51,8 @@ class SqliteDBExecutor(BaseSQLDBExecutor):
                                },
                                pool_pre_ping=True,
                                pool_recycle=1800)
-        logger.warning(f"Created ENGINE for high concurrency read settings but NO WRITE.")
-        return cls(engine=engine, cache_db=None)
+        logger.debug(f"Created ENGINE for high concurrency read settings but NO WRITE.")
+        return cls(engine=engine, cache_db=None, *args, **kwargs)
 
     def _install_pragmas_listener(self) -> None:
         """Attach a connect-time callback to *this* engine instance."""
@@ -187,7 +188,7 @@ class SqliteCacheDB(SqliteDBExecutor):
         if not os.path.exists(db_path):
             raise FileNotFoundError(f"SQLite database file not found at {db_path}")
         uri = f"sqlite:///{db_path}?mode=rwc&cache=shared&check_same_thread=false"
-        logger.info(f"Connecting to SQLite database with URI {uri}")
+        logger.debug(f"Connecting to SQLite database with URI {uri}")
         engine = create_engine(uri,
                                # echo=True, echo_pool=True,
                                pool_size=20, max_overflow=40,
@@ -197,7 +198,7 @@ class SqliteCacheDB(SqliteDBExecutor):
                                },
                                pool_pre_ping=True,
                                pool_recycle=1800)
-        initiated_object = cls(engine=engine, cache_db=None)
+        initiated_object = cls(engine=engine, *args, **kwargs)
         initiated_object.create_cache_table()
         return initiated_object
 
@@ -218,6 +219,7 @@ class SqliteCacheDB(SqliteDBExecutor):
                            """.strip()
         self.execute_query(create_table_sql)
 
+    @lru_cache(2000)
     def _is_id_already_present(self, hash_id: str) -> bool:
         stmt = "SELECT 1 FROM `cache_data` WHERE hash_key = :hash_id LIMIT 1"
         result_row = self.execute_query(query=stmt, params={"hash_id": hash_id})
@@ -234,7 +236,7 @@ class SqliteCacheDB(SqliteDBExecutor):
         hash_id = hash_db_id_sql(db_id, query)
 
         if self._is_id_already_present(hash_id):
-            self.logger.info('Skipping insert as already present in cache_db.')
+            self.logger.debug('Skipping insert as already present in cache_db.')
             return None
 
         self.logger.debug(
@@ -264,10 +266,14 @@ class SqliteCacheDB(SqliteDBExecutor):
             hash_id = hash_db_id_sql(db_id, parsed_query)
             pickled_result = pickle.dumps(result)
             if self._is_id_already_present(hash_id):
-                self.logger.info('Skipping insert as already present in cache_db.')
+                self.logger.debug('Skipping insert as already present in cache_db.')
                 continue
             to_insert.append({"hash_id": hash_id, "db_id": db_id, "query": parsed_query, "result": pickled_result})
 
+        if not to_insert:
+            self.logger.debug("Nothing to insert (all keys already present).")
+            return None
+
         stmt = "INSERT OR IGNORE INTO `cache_data` (hash_key, db_id, query, result) VALUES (:hash_id, :db_id, :query, :result)"
         try:
             with self.connection() as conn:
@@ -289,7 +295,7 @@ class SqliteCacheDB(SqliteDBExecutor):
         hash_id = hash_db_id_sql(db_id, query)
         self.logger.debug(f"Fetching from cache with hash_id: {hash_id}")
         if self.cache_db is not None and self.cache_db._is_id_already_present(hash_id):
-            self.logger.info('Fetching from cache_db as present in cache_db.')
+            self.logger.debug('Fetching from cache_db as present in cache_db.')
             return self.cache_db.fetch_from_cache(db_id, query)
 
         try:
@@ -311,9 +317,8 @@ class SqliteCacheDB(SqliteDBExecutor):
             return query
 
 
-
 if __name__ == "__main__":
     db = SqliteDBExecutor.from_uri(
-        relative_base_path='data/bird/train_databases/address/address.sqlite'
+        relative_base_path='data/bird_dev/dev_databases/california_schools/california_schools.sqlite',
     )
-    print(db.get_ddl_database(add_sample_rows_strategy='inline'))
\ No newline at end of file
+    print(db.get_ddl_database(add_sample_rows_strategy='inline', question='What is Augusta-Richmond County, GA-SC?'))
